/**
 * Js for the Advanced Sidebar Menu widgets
 *
 * @author Mat Lipe
 *
 */
/* global asm_pro_config */
var asm_pro = window.asm_pro || {
	accordionFields: {},
	styleFields: {},
	displayPosts: {},
	widgetId: null
};

(function ($, a, config) {
	/**
	 * Accordion fields
	 */
	a.accordionFields = {
		init: function() {
			$( a.widgetId + ' [data-js="advanced-sidebar-menu-pro/widget/accordion-style"]' )
				.on( 'change', this._change_style_display );

			var color = $( a.widgetId + ' [data-js="advanced-sidebar-menu-pro/widget/accordion-color"]' );
			color.on( 'change', this._change_color );
		},

		/**
		 * Change the icons displayed in the "Accordion icons style" field
		 * within the widget.
		 *
		 * @private
		 */
		_change_style_display: function () {
			var el = $(this);
			var styles = config.accordion_styles[el.val()];
			$( '[data-js="' + el.data( 'target' ) + '"]' )
				.html( styles.contract + ' ' + styles.expand ); //phpcs:ignore WordPressVIPMinimum.JS.HTMLExecutingFunctions.html
		},

		/**
		 * Change the accordion icon color in the widget settings.
		 *
		 * @since 8.2.0
		 *
		 * @param {Event} ev
		 * @param {object} values - { passedTrigger: boolean, color: string }
		 *
		 * @private
		 */
		_change_color: function( ev, values ) {
			if ( ! values || ! values.color ){
				return;
			}
			$( '[data-js="' + $( this ).data( 'target' ) + '"]' )
				.css( 'color', values.color );
		},
	};

	/**
	 * Display Posts select.
	 *
	 * @since 3.5.0
	 *
	 */
	a.displayPosts = {

		init: function () {
			var select = $(a.widgetId + ' [data-js="advanced-sidebar-menu-pro/widget-options/category/display-posts/levels"]');
			select.on( 'change',this._show_hide_fields);
		},
		/**
		 * Display the additional options if the select has a value.
		 * @private
		 */
		_show_hide_fields: function () {
			var select = $( this );
			var fieldWrap = $( '[id="' + select.data( 'target' ) + '"]' );
			if ( select.find( 'option:selected' ).val() !== '' ) {
				fieldWrap.show();
			} else {
				fieldWrap.hide();
			}
		}
	};

	/**
	 * Style fields
	 */
	a.styleFields = {
		init: function () {
			this._handle_style_fields();
			this._init_color_pickers();
		},

		/**
		 * So the widget may be saved when all we do is change
		 * a color from a picker.
		 *
		 * Need a timeout to allow the value to change before
		 * the change is fired.
		 *
		 * We use `passedTrigger` as a flag to prevent infinite loops
		 * caused by triggering 'change' during the change events.
		 *
		 * @link https://github.com/Automattic/Iris/issues/55#issuecomment-303716820
		 *
		 * @private
		 */
		_init_color_pickers() {
			var onChange = function( ev, ui ) {
				setTimeout( function() {
					if ( ui && ui.color && ! ui.passedTrigger ) {
						// Change color of accordion icons when applicable.
						$( ev.target ).trigger( 'change', {
							passedTrigger: true,
							color: ui.color.toString()
						} );
					}
				}, 1 );
			};

			$( a.widgetId + ' .asm-color-select' ).wpColorPicker( {
				change: onChange,
				clear: function( ev ) {
					/**
					 * isTrigger will be set for non interactive clears which
					 * cause and infinite loop.
					 */
					if ( ! ev.isTrigger ) {
						ev.target = $( ev.target ).parent().find('.wp-color-picker')
						onChange( ev, {
							color: 'inherit',
						} );
					}
				},
			} );
		},

		/**
		 * Show/Display the style fields.
		 * Change the button text depending on state.
		 *
		 * @private
		 */
		_handle_style_fields: function () {
			$(a.widgetId + ' [data-js="advanced-sidebar-menu-pro/widget/show-styles"]').click( function ( ev ) {
				ev.preventDefault();
				var el = $(this);
				var target = $('[data-js="' + el.data('target') + '"]');
				target.slideToggle(function () {
					if (target.is(':visible')) {
						el.parent().find( '[data-js="advanced-sidebar-menu-pro/widget/show-styles/status"]' ).val( 0 );
						el.text(config.i18n.hide_styles);
					} else {
						el.parent().find( '[data-js="advanced-sidebar-menu-pro/widget/show-styles/status"]' ).val( 1 );
						el.text(config.i18n.show_styles);
					}
				});
			});
		},
	};

	/**
	 * Triggered via inline js to account for the form
	 * updates via ajax as WP saves and adds widgets
	 */
	a.init = function (widgetId) {
		a.widgetId = widgetId;
		a.styleFields.init();
		a.accordionFields.init();
		a.displayPosts.init();
	};

})(jQuery, asm_pro, asm_pro_config);
