<?php

namespace Advanced_Sidebar_Menu;

use Advanced_Sidebar_Menu\Traits\Singleton;
use Advanced_Sidebar_Menu\Widget_Options\Shared\Accordion;

/**
 * Script and style handling for all widgets.
 *
 * @author OnPoint Plugins
 */
class Pro_Scripts {
	use Singleton;

	const HANDLE = 'asm-widgets';

	/**
	 * Register actions and filters.
	 */
	public function hook() {
		add_action( 'advanced-sidebar-menu/widget/page/after-form', [ $this, 'widget_scripts' ] );

		// UGH! beaver builder hack.
		if ( isset( $_GET['fl_builder'] ) ) { // phpcs:ignore
			add_action( 'wp_enqueue_scripts', [ $this, 'load_scripts_on_front_end' ] );
		}
		// Elementor support.
		add_action( 'elementor/editor/after_enqueue_scripts', [ $this, 'load_scripts_on_front_end' ] );
	}


	/**
	 * Cue the JS for the front end.
	 *
	 * @notice Only call when needed
	 *
	 * @since  3.10.0
	 *
	 * @return void
	 */
	public function frontend_scripts() {
		wp_enqueue_script( 'asm-pro-front-end', ADVANCED_SIDEBAR_MENU_PRO_URL . 'assets/js/front-end.js', [ 'jquery' ], ADVANCED_SIDEBAR_MENU_PRO_VERSION, true );
		wp_localize_script( 'asm-pro-front-end', 'asm_pro_config', $this->js_config() );
	}


	/**
	 * Beaver builder and Elementor require scripts
	 * to be loaded on the front-end.
	 * This causes conflicts with color-pickers etc.
	 *
	 * @todo if additional use cases start creeping up, build something more
	 *       robust for handling this
	 *
	 * @return void
	 */
	public function load_scripts_on_front_end() {
		$scripts = wp_scripts();
		$suffix = wp_scripts_get_suffix();
		// @see wp/wp-includes/script-loader.php:1275
		$scripts->add( 'iris', '/wp-admin/js/iris.min.js', [
			'jquery-ui-draggable',
			'jquery-ui-slider',
			'jquery-touch-punch',
		], '1.0.7', 1 );
		$scripts->add( 'wp-color-picker', "/wp-admin/js/color-picker{$suffix}.js", [ 'iris' ], false, 1 );
		$scripts->set_translations( 'wp-color-picker', 'default' );

		// This may be removed with minimum version is WP 5.5+.
		//phpcs:disable
		did_action( 'init' ) && $scripts->localize(
			'wp-color-picker',
			'wpColorPickerL10n',
			[
				'clear'            => __( 'Clear' ),
				'clearAriaLabel'   => __( 'Clear color' ),
				'defaultString'    => __( 'Default' ),
				'defaultAriaLabel' => __( 'Select default color' ),
				'pick'             => __( 'Select Color' ),
				'defaultLabel'     => __( 'Color value' ),
			]
		);
		//phpcs:enable

		Scripts::instance()->admin_scripts();

		$this->widget_scripts();
	}


	/**
	 * Only call when needed
	 *
	 * @return void
	 */
	public function widget_scripts() {
		wp_enqueue_style( 'wp-color-picker' );
		wp_enqueue_script( 'wp-color-picker' );

		wp_enqueue_script( self::HANDLE, ADVANCED_SIDEBAR_MENU_PRO_URL . 'assets/js/asm-widgets.js', [ 'underscore' ], ADVANCED_SIDEBAR_MENU_PRO_VERSION, true );
		wp_localize_script( self::HANDLE, 'asm_pro_config', $this->js_config() );
	}


	/**
	 * Configuration passed from PHP to JavaScript.
	 *
	 * @return array
	 */
	private function js_config() {
		return [
			'accordion_styles'    => Accordion::instance()->get_accordion_style_options(),
			'accordion_style_key' => Accordion::ACCORDION_STYLE,
			'i18n'                => [
				'show_styles' => __( 'Click to show styles', 'advanced-sidebar-menu' ),
				'hide_styles' => __( 'Click to hide styles', 'advanced-sidebar-menu' ),
			],
		];
	}

}
