<?php

namespace Advanced_Sidebar_Menu\Widget_Options\Shared;

use Advanced_Sidebar_Menu\Factory;
use Advanced_Sidebar_Menu\Menus\Menu_Abstract;
use Advanced_Sidebar_Menu\Traits\Singleton;
use Advanced_Sidebar_Menu\Widget\Widget_Abstract;
use Advanced_Sidebar_Menu\Widget_Options\Widget_Options_Abstract;

/**
 * Style options and CSS rendering for all widgets.
 *
 * @since  3.3.0
 */
class Styles extends Widget_Options_Abstract {
	use Singleton;

	const NAME = true;

	const STYLES_OPEN = 'styles-open';

	/**
	 * Fields for the widget.
	 *
	 * @var array
	 */
	protected $fields = [];


	/**
	 * Styles constructor.
	 */
	public function __construct() {
		$this->fields = [
			[
				'label'  => __( 'Current Item', 'advanced-sidebar-menu' ),
				'prefix' => 'current_page',
			],
			[
				'label'  => __( 'Current Item Parent', 'advanced-sidebar-menu' ),
				'prefix' => 'current_page_parent',
			],
			[
				'label'  => __( 'Highest Level Parent', 'advanced-sidebar-menu' ),
				'prefix' => 'parent_page',
			],
			[
				'label'  => __( 'Child', 'advanced-sidebar-menu' ),
				'prefix' => 'child_page',
			],
			[
				'label'  => __( 'Grandchild', 'advanced-sidebar-menu' ),
				'prefix' => 'grandchild_page',
			],
		];
	}


	/**
	 * Actions.
	 */
	protected function hook() {
		add_action( 'advanced-sidebar-menu/menus/page/render', [ $this, 'render' ] );
		add_action( 'advanced-sidebar-menu/widget/page/after-form', [ $this, 'render_fields' ], 10, 2 );

		add_action( 'advanced-sidebar-menu/menus/category/render', [ $this, 'render' ] );
		add_action( 'advanced-sidebar-menu/widget/category/after-form', [ $this, 'render_fields' ], 10, 2 );

		add_action( 'advanced-sidebar-menu/menus/navigation/render', [ $this, 'render' ] );
		add_action( 'advanced-sidebar-menu/widget/navigation-menu/after-form', [ $this, 'render_fields' ], 10, 2 );
	}


	/**
	 * Output the styles
	 *
	 * @param Menu_Abstract $menu - Current menu.
	 *
	 * @return void
	 */
	public function render( Menu_Abstract $menu ) {
		$widget_args = $menu->get_widget_args();
		$instance = $menu->get_widget_instance();
		$out = null;

		// The CSS id.
		$css_id = '#' . $widget_args['widget_id'] . Factory::CSS_SELECTOR_APPEND . ' ';

		$styles = [];
		foreach ( wp_list_pluck( $this->fields, 'prefix' ) as $section ) {
			$styles[ $section ] = [
				'default' => '',
				'hover'   => '',
			];

			if ( ! empty( $instance[ $section . '_color' ] ) ) {
				$styles[ $section ]['default'] .= 'color: ' . $instance[ $section . '_color' ] . ' !important;';
			}

			if ( ! empty( $instance[ $section . '_bg_color' ] ) ) {
				$styles[ $section ]['default'] .= 'background-color: ' . $instance[ $section . '_bg_color' ] . ' !important; ';
			}

			if ( ! empty( $instance[ $section . '_font_weight' ] ) ) {
				$styles[ $section ]['default'] .= 'font-weight: ' . $instance[ $section . '_font_weight' ] . ' !important; ';
			}

			if ( ! empty( $instance[ $section . '_font_size' ] ) ) {
				$styles[ $section ]['default'] .= 'font-size: ' . $instance[ $section . '_font_size' ] . 'px !important;';
			}

			if ( ! empty( $instance[ $section . '_hover_color' ] ) ) {
				$styles[ $section ]['hover'] .= 'color: ' . $instance[ $section . '_hover_color' ] . ' !important;';
			}

			if ( ! empty( $instance[ $section . '_hover_bg_color' ] ) ) {
				$styles[ $section ]['hover'] .= 'background-color: ' . $instance[ $section . '_hover_bg_color' ] . ' !important; ';
			}
		}

		$styles = apply_filters( 'advanced-sidebar-menu-pro/widget-options/shared/styles/styles', $styles, $menu, $css_id );

		if ( ! empty( $instance[ Style_Targeting::BULLET_STYLE ] ) ) {
			$out .= $css_id . 'li{ list-style-type: ' . $instance[ Style_Targeting::BULLET_STYLE ] . ' !important; list-style: ' . $instance['bullet_style'] . '!important;}';
		}

		if ( ! empty( $styles['parent_page']['default'] ) ) {
			$out .= $css_id . '.parent-sidebar-menu > li > a{' . $styles['parent_page']['default'] . '}';
		}
		if ( ! empty( $styles['parent_page']['hover'] ) ) {
			$out .= $css_id . '.parent-sidebar-menu > li > a:hover{' . $styles['parent_page']['hover'] . '}';
		}

		if ( ! empty( $styles['child_page']['default'] ) ) {
			$out .= $css_id . '.child-sidebar-menu > li > a{' . $styles['child_page']['default'] . '}';
		}
		if ( ! empty( $styles['child_page']['hover'] ) ) {
			$out .= $css_id . '.child-sidebar-menu > li > a:hover{' . $styles['child_page']['hover'] . '}';
		}

		if ( ! empty( $styles['grandchild_page']['default'] ) ) {
			$out .= $css_id . '.child-sidebar-menu .children li a{' . $styles['grandchild_page']['default'] . '}';
		}
		if ( ! empty( $styles['grandchild_page']['hover'] ) ) {
			$out .= $css_id . '.child-sidebar-menu .children li a:hover{' . $styles['grandchild_page']['hover'] . '}';
		}

		if ( ! empty( $styles['current_page']['default'] ) ) {
			if ( ! $menu->checked( Style_Targeting::APPLY_CURRENT_STYLES ) ) {
				$out .= $css_id . '.child-sidebar-menu li.page_item.current_page_item > a{' . $styles['current_page']['default'] . '}';
				$out .= $css_id . '.child-sidebar-menu li.menu-item.current-menu-item > a{' . $styles['current_page']['default'] . '}';
				$out .= $css_id . '.child-sidebar-menu li.cat-item.current-cat > a{' . $styles['current_page']['default'] . '}';
			} else {
				$out .= $css_id . 'li.page_item.current_page_item > a{' . $styles['current_page']['default'] . '}';
				$out .= $css_id . 'li.menu-item.current-menu-item > a{' . $styles['current_page']['default'] . '}';
				$out .= $css_id . 'li.cat-item.current-cat > a{' . $styles['current_page']['default'] . '}';
			}
		}
		if ( ! empty( $styles['current_page']['hover'] ) ) {
			if ( ! $menu->checked( Style_Targeting::APPLY_CURRENT_STYLES ) ) {
				$out .= $css_id . '.child-sidebar-menu li.page_item.current_page_item > a:hover{' . $styles['current_page']['hover'] . '}';
				$out .= $css_id . '.child-sidebar-menu li.menu-item.current-menu-item > a:hover{' . $styles['current_page']['hover'] . '}';
				$out .= $css_id . '.child-sidebar-menu li.cat-item.current-cat > a:hover{' . $styles['current_page']['hover'] . '}';
			} else {
				$out .= $css_id . 'li.page_item.current_page_item > a:hover{' . $styles['current_page']['hover'] . '}';
				$out .= $css_id . 'li.menu-item.current-menu-item > a:hover{' . $styles['current_page']['hover'] . '}';
				$out .= $css_id . 'li.cat-item.current-cat > a:hover{' . $styles['current_page']['hover'] . '}';
			}
		}

		if ( ! empty( $styles['current_page_parent']['default'] ) ) {
			if ( ! $menu->checked( Style_Targeting::APPLY_PARENT_STYLES ) ) {
				$out .= $css_id . '.child-sidebar-menu li.page_item.current_page_parent > a{' . $styles['current_page_parent']['default'] . '}';
				$out .= $css_id . '.child-sidebar-menu li.menu-item.current-menu-parent > a{' . $styles['current_page_parent']['default'] . '}';
				$out .= $css_id . '.child-sidebar-menu li.cat-item.current-cat-parent > a{' . $styles['current_page_parent']['default'] . '}';
			} else {
				$out .= $css_id . 'li.page_item.current_page_parent > a{' . $styles['current_page_parent']['default'] . '}';
				$out .= $css_id . 'li.menu-item.current-menu-parent > a{' . $styles['current_page_parent']['default'] . '}';
				$out .= $css_id . 'li.cat-item.current-cat-parent > a{' . $styles['current_page_parent']['default'] . '}';
			}
		}
		if ( ! empty( $styles['current_page_parent']['hover'] ) ) {
			if ( ! $menu->checked( Style_Targeting::APPLY_PARENT_STYLES ) ) {
				$out .= $css_id . '.child-sidebar-menu li.page_item.current_page_parent > a:hover{' . $styles['current_page_parent']['hover'] . '}';
				$out .= $css_id . '.child-sidebar-menu li.menu-item.current-menu-parent > a:hover{' . $styles['current_page_parent']['hover'] . '}';
				$out .= $css_id . '.child-sidebar-menu li.cat-item.current-cat-parent > a:hover{' . $styles['current_page_parent']['hover'] . '}';
			} else {
				$out .= $css_id . 'li.page_item.current_page_parent > a:hover{' . $styles['current_page_parent']['hover'] . '}';
				$out .= $css_id . 'li.menu-item.current-menu-parent > a:hover{' . $styles['current_page_parent']['hover'] . '}';
				$out .= $css_id . 'li.cat-item.current-cat-parent > a:hover{' . $styles['current_page_parent']['hover'] . '}';
			}
		}

		if ( $menu->checked( Style_Targeting::BLOCK_STYLE ) ) {
			$out .= Style_Targeting::instance()->get_block_styles( $css_id, $instance, $menu );
			if ( $menu->checked( Style_Targeting::BORDER ) ) {
				$out .= Style_Targeting::instance()->get_border_styles( $css_id, $instance );
			}
		}

		$out = apply_filters( 'advanced-sidebar-menu-pro/widget-options/shared/styles/output', $out, $styles, $menu, $css_id );

		if ( null !== $out ) {
			printf( "<style id='%s-inline-css' type='text/css'>\n%s\n</style>\n", esc_attr( $widget_args['widget_id'] ), $out ); //phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
		}
	}


	/**
	 * List of available font weights.
	 *
	 * @return array
	 */
	public function get_font_weights() {
		$weights = [
			'100' => __( 'Thin', 'advanced-sidebar-menu' ),
			'200' => __( 'Extra Light', 'advanced-sidebar-menu' ),
			'300' => __( 'Light', 'advanced-sidebar-menu' ),
			'400' => __( 'Normal', 'advanced-sidebar-menu' ),
			'500' => __( 'Medium', 'advanced-sidebar-menu' ),
			'600' => __( 'Semi Bold', 'advanced-sidebar-menu' ),
			'700' => __( 'Bold', 'advanced-sidebar-menu' ),
			'800' => __( 'Extra Bold', 'advanced-sidebar-menu' ),
			'900' => __( 'Black', 'advanced-sidebar-menu' ),
			'950' => __( 'Extra Black', 'advanced-sidebar-menu' ),
		];

		return apply_filters( 'advanced-sidebar-menu/styles/font-weights', $weights );
	}


	/**
	 * Translate the old string based font weights
	 * to their equivalent numeric value.
	 *
	 * @notice Will be removed after 6/1/2021.
	 *
	 * @param string $font_weight - Current value for font weight.
	 *
	 * @internal
	 * @since 8.2.0
	 *
	 * @todo Remove after 6/1/2021.
	 *
	 * @return string|string[]
	 */
	protected function translate_old_font_weight_values( $font_weight ) {
		return str_replace( [
			'normal',
			'bold',
			'lighter',
		], [
			'400',
			'700',
			'300',
		], $font_weight );
	}


	/**
	 * Detect if we are currently saving this widget and whether
	 * the styles should be closed or open when the form loads.
	 *
	 * Allows keeping the styles open after save if they were before saving.
	 *
	 * @param Widget_Abstract $widget - Current widget.
	 *
	 * @see   wp_ajax_save_widget
	 *
	 * @since 8.2.0
	 *
	 * @return bool
	 */
	protected function is_styles_closed( Widget_Abstract $widget ) {
		//phpcs:disable
		$closed = true;
		if (
			! empty( $_REQUEST['widget-id'] ) &&
			! empty( $_REQUEST["widget-{$widget->id_base}"] ) &&
			$widget->id === sanitize_text_field( $_REQUEST['widget-id'] ) ) {
			$current = reset( $_REQUEST["widget-{$widget->id_base}"] );
			if ( isset( $current['styles-open'] ) ) {
				$closed = (bool) $current[ static::STYLES_OPEN ];
			}
		}
		//phpcs:enable

		return $closed;
	}


	/**
	 * Render the widget fields.
	 *
	 * @param array           $instance - Widget settings.
	 * @param Widget_Abstract $widget   - Widget class.
	 *
	 * @return mixed|void
	 */
	public function render_fields( array $instance, Widget_Abstract $widget ) {
		$closed = $this->is_styles_closed( $widget );
		?>
		<div
			class="advanced-sidebar-menu-full-width"
			style="display:none;"
			data-advanced-sidebar-menu-hide="0">
			<div class="advanced-sidebar-menu-column-box advanced-sidebar-menu-styles-button">
				<!-- Holds the open/closed state so we can retain after save -->
				<input
					data-js="advanced-sidebar-menu-pro/widget/show-styles/status"
					type="hidden"
					name="<?php echo esc_attr( $widget->get_field_name( static::STYLES_OPEN ) ); ?>"
					value="<?php echo (int) $closed; ?>" />
				<button
					class="button-secondary widefat"
					data-js="advanced-sidebar-menu-pro/widget/show-styles"
					data-target="<?php echo esc_js( $widget->id ); ?>-css">
					<?php
					$closed ? esc_html_e( 'Click to show styles', 'advanced-sidebar-menu' ) : esc_html_e( 'Click to hide styles', 'advanced-sidebar-menu' );
					?>
				</button>
			</div>
		</div>

		<div
			data-js="<?php echo esc_attr( $widget->id ); ?>-css"
			data-advanced-sidebar-menu-hide="<?php echo (int) $closed; ?>">
			<div class="advanced-sidebar-menu-column">
				<?php
				Style_Targeting::instance()->render_box( $instance, $widget );

				foreach ( array_slice( $this->fields, 0, 2 ) as $section => $_fields ) {
					$this->section_settings( $_fields, $instance, $widget );
				}
				?>
			</div>
			<div class="advanced-sidebar-menu-column advanced-sidebar-menu-column-right">
				<?php
				foreach ( array_slice( $this->fields, 2 ) as $section => $_fields ) {
					$this->section_settings( $_fields, $instance, $widget );
				}
				?>
			</div>
			<div class="advanced-sidebar-menu-full-width"><!-- Clear --></div>
		</div>
		<?php
	}


	/**
	 * Render the color settings for a section.
	 *
	 * @param array           $section  - List of settings.
	 * @param array           $instance - Widget settings.
	 * @param Widget_Abstract $widget   - Widget class.
	 *
	 * @since 8.2.0
	 */
	public function section_settings( array $section, array $instance, Widget_Abstract $widget ) {
		$fields = [
			"{$section['prefix']}_color"          => __( 'Color', 'advanced-sidebar-menu' ),
			"{$section['prefix']}_bg_color"       => __( 'Background', 'advanced-sidebar-menu' ),
			"{$section['prefix']}_font_weight"    => __( 'Font weight', 'advanced-sidebar-menu' ),
			"{$section['prefix']}_font_size"      => __( 'Font size', 'advanced-sidebar-menu' ) . ' (px)',
			"{$section['prefix']}_hover_color"    => __( 'Hover color', 'advanced-sidebar-menu' ),
			"{$section['prefix']}_hover_bg_color" => __( 'Hover background', 'advanced-sidebar-menu' ),
		];

		foreach ( array_keys( $fields ) as $_key ) {
			if ( ! isset( $instance[ $_key ] ) ) {
				$instance[ $_key ] = false;
			}
		}

		// Translate old font weight values to numbers (remove after 6/1/2021).
		$instance[ $section['prefix'] . '_font_weight' ] = $this->translate_old_font_weight_values( $instance[ $section['prefix'] . '_font_weight' ] );

		?>
		<div class="advanced-sidebar-menu-column-box advanced-sidebar-menu-styles-box">
			<h3>
				<?php echo esc_html( $section['label'] ); ?>
			</h3>
			<?php

			foreach ( $fields as $_field => $_label ) {
				if ( 'label' === $_field ) {
					continue;
				}
				?>

				<div class="advanced-sidebar-menu-column">
					<label for="<?php echo esc_attr( $widget->get_field_id( $_field ) ); ?>">
						<?php echo esc_html( $_label ); ?>
					</label>
				</div>
				<div class="advanced-sidebar-menu-column advanced-sidebar-menu-column-right">

					<?php
					// Font weights.
					if ( strpos( $_field, 'weight' ) !== false ) {
						?>
						<select
							name="<?php echo esc_attr( $widget->get_field_name( $_field ) ); ?>"
							id="<?php echo esc_attr( $widget->get_field_id( $_field ) ); ?>">
							<option value="">
								<?php esc_html_e( '- default -', 'advanced-sidebar-menu' ); ?>
							</option>
							<?php
							foreach ( $this->get_font_weights() as $weight => $__label ) {
								printf( '<option value="%s" %s>%s</option>', esc_attr( $weight ), selected( $instance[ $_field ], $weight ), esc_html( $__label ) );
							}
							?>
						</select>
						<?php
						// Colors.
					} elseif ( strpos( $_field, 'color' ) !== false ) {
						printf( '<input type="text" name="%s" value="%s" id="%s" class="asm-color-select">', esc_attr( $widget->get_field_name( $_field ) ), esc_html( $instance[ $_field ] ), esc_attr( $widget->get_field_id( $_field ) ) );
						// Size.
					} elseif ( strpos( $_field, 'size' ) !== false ) {
						printf( '<input type="number" step="1" min="2" name="%s" value="%s" id="%s" size="5">', esc_attr( $widget->get_field_name( $_field ) ), esc_html( $instance[ $_field ] ), esc_attr( $widget->get_field_id( $_field ) ) );
					}
					?>
				</div>

				<?php
			}
			?>
			<div class="advanced-sidebar-menu-full-width"><!-- Clear --></div>
		</div>
		<?php
	}
}
